/**
 * chroma.js - JavaScript library for color conversions
 *
 * Copyright (c) 2011-2024, Gregor Aisch
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name Gregor Aisch may not be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL GREGOR AISCH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * -------------------------------------------------------
 *
 * chroma.js includes colors from colorbrewer2.org, which are released under
 * the following license:
 *
 * Copyright (c) 2002 Cynthia Brewer, Mark Harrower,
 * and The Pennsylvania State University.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * ------------------------------------------------------
 *
 * Named colors are taken from X11 Color Names.
 * http://www.w3.org/TR/css3-color/#svg-color
 *
 * @preserve
 */

!function(t,r){"object"==typeof exports&&"undefined"!=typeof module?r(exports):"function"==typeof define&&define.amd?define(["exports"],r):r((t="undefined"!=typeof globalThis?globalThis:t||self).chroma={})}(this,(function(t){"use strict";function r(t,r,n){return void 0===n&&(n=1),s(c(r,t),n)}for(var n={},e=0,o=["Boolean","Number","String","Function","Array","Date","RegExp","Undefined","Null"];e<o.length;e+=1){var a=o[e];n["[object "+a+"]"]=a.toLowerCase()}function i(t){return n[Object.prototype.toString.call(t)]||"object"}function u(t,r){return void 0===r&&(r=null),t.length>=3?Array.prototype.slice.call(t):"object"==i(t[0])&&r?r.split("").filter((function(r){return void 0!==t[0][r]})).map((function(r){return t[0][r]})):t[0]}function l(t){if(t.length<2)return null;var r=t.length-1;return"string"==i(t[r])?t[r].toLowerCase():null}var s=Math.min,c=Math.max,h={format:{},autodetect:[]},f=function(){for(var t=[],n=arguments.length;n--;)t[n]=arguments[n];var e=this;if("object"===i(t[0])&&t[0].constructor&&t[0].constructor===this.constructor)return t[0];var o=l(t),a=!1;if(!o){a=!0,h.sorted||(h.autodetect=h.autodetect.sort((function(t,r){return r.p-t.p})),h.sorted=!0);for(var u=0,s=h.autodetect;u<s.length;u+=1){var c=s[u];if(o=c.test.apply(c,t))break}}if(!h.format[o])throw new Error("unknown format: "+t);var f=h.format[o].apply(null,a?t:t.slice(0,-1));e._rgb=function(t){t._clipped=!1,t._unclipped=t.slice(0);for(var n=0;n<=3;n++)n<3?((t[n]<0||t[n]>255)&&(t._clipped=!0),t[n]=r(t[n],0,255)):3===n&&(t[n]=r(t[n],0,1));return t}(f),3===e._rgb.length&&e._rgb.push(1)};f.prototype.toString=function(){return"function"==i(this.hex)?this.hex():"["+this._rgb.join(",")+"]"};var p=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];return new(Function.prototype.bind.apply(f,[null].concat(t)))};p.version="2.6.1-0";var g=function(t){return Math.round(100*t)/100},b=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n,e,o=(t=u(t,"rgba"))[0],a=t[1],i=t[2],l=s(o/=255,a/=255,i/=255),h=c(o,a,i),f=(h+l)/2;return h===l?(n=0,e=Number.NaN):n=f<.5?(h-l)/(h+l):(h-l)/(2-h-l),o==h?e=(a-i)/(h-l):a==h?e=2+(i-o)/(h-l):i==h&&(e=4+(o-a)/(h-l)),(e*=60)<0&&(e+=360),t.length>3&&void 0!==t[3]?[e,n,f,t[3]]:[e,n,f]},d=Math.round,v=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=u(t,"rgba"),e=l(t)||"rgb";return"hsl"==e.substr(0,3)?function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=u(t,"hsla"),e=l(t)||"lsa";return n[0]=g(n[0]||0),n[1]=g(100*n[1])+"%",n[2]=g(100*n[2])+"%","hsla"===e||n.length>3&&n[3]<1?(n[3]=n.length>3?n[3]:1,e="hsla"):n.length=3,e+"("+n.join(",")+")"}(b(n),e):(n[0]=d(n[0]),n[1]=d(n[1]),n[2]=d(n[2]),("rgba"===e||n.length>3&&n[3]<1)&&(n[3]=n.length>3?n[3]:1,e="rgba"),e+"("+n.slice(0,"rgb"===e?3:4).join(",")+")")},y=function(){for(var t,r=[],n=arguments.length;n--;)r[n]=arguments[n];var e,o,a,i=(r=u(r,"hsl"))[0],l=r[1],s=r[2];if(0===l)e=o=a=255*s;else{var c=[0,0,0],h=[0,0,0],f=s<.5?s*(1+l):s+l-s*l,p=2*s-f,g=i/360;c[0]=g+1/3,c[1]=g,c[2]=g-1/3;for(var b=0;b<3;b++)c[b]<0&&(c[b]+=1),c[b]>1&&(c[b]-=1),6*c[b]<1?h[b]=p+6*(f-p)*c[b]:2*c[b]<1?h[b]=f:3*c[b]<2?h[b]=p+(f-p)*(2/3-c[b])*6:h[b]=p;e=(t=[255*h[0],255*h[1],255*h[2]])[0],o=t[1],a=t[2]}return r.length>3?[e,o,a,r[3]]:[e,o,a,1]},w=/^rgb\(\s*(-?\d+),\s*(-?\d+)\s*,\s*(-?\d+)\s*\)$/,m=/^rgba\(\s*(-?\d+),\s*(-?\d+)\s*,\s*(-?\d+)\s*,\s*([01]|[01]?\.\d+)\)$/,k=/^rgb\(\s*(-?\d+(?:\.\d+)?)%,\s*(-?\d+(?:\.\d+)?)%\s*,\s*(-?\d+(?:\.\d+)?)%\s*\)$/,_=/^rgba\(\s*(-?\d+(?:\.\d+)?)%,\s*(-?\d+(?:\.\d+)?)%\s*,\s*(-?\d+(?:\.\d+)?)%\s*,\s*([01]|[01]?\.\d+)\)$/,M=/^hsl\(\s*(-?\d+(?:\.\d+)?),\s*(-?\d+(?:\.\d+)?)%\s*,\s*(-?\d+(?:\.\d+)?)%\s*\)$/,x=/^hsla\(\s*(-?\d+(?:\.\d+)?),\s*(-?\d+(?:\.\d+)?)%\s*,\s*(-?\d+(?:\.\d+)?)%\s*,\s*([01]|[01]?\.\d+)\)$/,j=Math.round,F=function(t){var r;if(t=t.toLowerCase().trim(),h.format.named)try{return h.format.named(t)}catch(t){}if(r=t.match(w)){for(var n=r.slice(1,4),e=0;e<3;e++)n[e]=+n[e];return n[3]=1,n}if(r=t.match(m)){for(var o=r.slice(1,5),a=0;a<4;a++)o[a]=+o[a];return o}if(r=t.match(k)){for(var i=r.slice(1,4),u=0;u<3;u++)i[u]=j(2.55*i[u]);return i[3]=1,i}if(r=t.match(_)){for(var l=r.slice(1,5),s=0;s<3;s++)l[s]=j(2.55*l[s]);return l[3]=+l[3],l}if(r=t.match(M)){var c=r.slice(1,4);c[1]*=.01,c[2]*=.01;for(var f=y(c),p=0;p<3;p++)f[p]=j(f[p]);return f[3]=1,f}if(r=t.match(x)){var g=r.slice(1,4);g[1]*=.01,g[2]*=.01;for(var b=y(g),d=0;d<3;d++)b[d]=j(b[d]);return b[3]=+r[4],b}};F.test=function(t){return w.test(t)||m.test(t)||k.test(t)||_.test(t)||M.test(t)||x.test(t)},f.prototype.css=function(t){return v(this._rgb,t)},p.css=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];return new(Function.prototype.bind.apply(f,[null].concat(t,["css"])))},h.format.css=F,h.autodetect.push({p:5,test:function(t){for(var r=[],n=arguments.length-1;n-- >0;)r[n]=arguments[n+1];if(!r.length&&"string"===i(t)&&F.test(t))return"css"}});var N=/^#?([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$/,$=/^#?([A-Fa-f0-9]{8}|[A-Fa-f0-9]{4})$/,A=Math.round;f.prototype.hex=function(t){return function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=u(t,"rgba"),e=n[0],o=n[1],a=n[2],i=n[3],s=l(t)||"auto";void 0===i&&(i=1),"auto"===s&&(s=i<1?"rgba":"rgb");var c="000000"+((e=A(e))<<16|(o=A(o))<<8|(a=A(a))).toString(16);c=c.substr(c.length-6);var h="0"+A(255*i).toString(16);switch(h=h.substr(h.length-2),s.toLowerCase()){case"rgba":return"#"+c+h;case"argb":return"#"+h+c;default:return"#"+c}}(this._rgb,t)},p.hex=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];return new(Function.prototype.bind.apply(f,[null].concat(t,["hex"])))},h.format.hex=function(t){if(t.match(N)){4!==t.length&&7!==t.length||(t=t.substr(1)),3===t.length&&(t=(t=t.split(""))[0]+t[0]+t[1]+t[1]+t[2]+t[2]);var r=parseInt(t,16);return[r>>16,r>>8&255,255&r,1]}if(t.match($)){5!==t.length&&9!==t.length||(t=t.substr(1)),4===t.length&&(t=(t=t.split(""))[0]+t[0]+t[1]+t[1]+t[2]+t[2]+t[3]+t[3]);var n=parseInt(t,16);return[n>>24&255,n>>16&255,n>>8&255,Math.round((255&n)/255*100)/100]}throw new Error("unknown hex color: "+t)},h.autodetect.push({p:4,test:function(t){for(var r=[],n=arguments.length-1;n-- >0;)r[n]=arguments[n+1];if(!r.length&&"string"===i(t)&&[3,4,5,6,7,8,9].indexOf(t.length)>=0)return"hex"}}),f.prototype.hsl=function(){return b(this._rgb)},p.hsl=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];return new(Function.prototype.bind.apply(f,[null].concat(t,["hsl"])))},h.format.hsl=y,h.autodetect.push({p:2,test:function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];if("array"===i(t=u(t,"hsl"))&&3===t.length)return"hsl"}});var C=18,E=.95047,O=1,S=1.08883,L=.137931034,I=.206896552,T=.12841855,q=.008856452,B=Math.pow,D=function(t){return 255*(t<=.00304?12.92*t:1.055*B(t,1/2.4)-.055)},P=function(t){return t>I?t*t*t:T*(t-L)},R=Math.pow,U=function(t){return(t/=255)<=.04045?t/12.92:R((t+.055)/1.055,2.4)},z=function(t){return t>q?R(t,1/3):t/T+L},G=function(t,r,n){return t=U(t),r=U(r),n=U(n),[z((.4124564*t+.3575761*r+.1804375*n)/E),z((.2126729*t+.7151522*r+.072175*n)/O),z((.0193339*t+.119192*r+.9503041*n)/S)]};f.prototype.lab=function(){return function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=u(t,"rgb"),e=n[0],o=n[1],a=n[2],i=G(e,o,a),l=i[0],s=i[1],c=116*s-16;return[c<0?0:c,500*(l-s),200*(s-i[2])]}(this._rgb)},p.lab=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];return new(Function.prototype.bind.apply(f,[null].concat(t,["lab"])))},h.format.lab=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n,e,o,a=(t=u(t,"lab"))[0],i=t[1],l=t[2];return e=(a+16)/116,n=isNaN(i)?e:e+i/500,o=isNaN(l)?e:e-l/200,e=O*P(e),n=E*P(n),o=S*P(o),[D(3.2404542*n-1.5371385*e-.4985314*o),D(-.969266*n+1.8760108*e+.041556*o),D(.0556434*n-.2040259*e+1.0572252*o),t.length>3?t[3]:1]},h.autodetect.push({p:2,test:function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];if("array"===i(t=u(t,"lab"))&&3===t.length)return"lab"}});var H=Math.pow,J=Math.sign;function K(t){var r=Math.abs(t);return r>.0031308?(J(t)||1)*(1.055*H(r,1/2.4)-.055):12.92*t}var Q=Math.cbrt,V=Math.pow,W=Math.sign;function X(t){var r=Math.abs(t);return r<.04045?t/12.92:(W(t)||1)*V((r+.055)/1.055,2.4)}f.prototype.oklab=function(){return function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=u(t,"rgb"),e=n[0],o=n[1],a=n[2],i=[X(e/255),X(o/255),X(a/255)],l=i[0],s=i[1],c=i[2],h=Q(.4122214708*l+.5363325363*s+.0514459929*c),f=Q(.2119034982*l+.6806995451*s+.1073969566*c),p=Q(.0883024619*l+.2817188376*s+.6299787005*c);return[.2104542553*h+.793617785*f-.0040720468*p,1.9779984951*h-2.428592205*f+.4505937099*p,.0259040371*h+.7827717662*f-.808675766*p]}(this._rgb)},p.oklab=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];return new(Function.prototype.bind.apply(f,[null].concat(t,["oklab"])))},h.format.oklab=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=(t=u(t,"lab"))[0],e=t[1],o=t[2],a=H(n+.3963377774*e+.2158037573*o,3),i=H(n-.1055613458*e-.0638541728*o,3),l=H(n-.0894841775*e-1.291485548*o,3);return[255*K(4.0767416621*a-3.3077115913*i+.2309699292*l),255*K(-1.2684380046*a+2.6097574011*i-.3413193965*l),255*K(-.0041960863*a-.7034186147*i+1.707614701*l),t.length>3?t[3]:1]},h.autodetect.push({p:3,test:function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];if("array"===i(t=u(t,"oklab"))&&3===t.length)return"oklab"}});var Y=Math.round;f.prototype.rgb=function(t){return void 0===t&&(t=!0),!1===t?this._rgb.slice(0,3):this._rgb.slice(0,3).map(Y)},f.prototype.rgba=function(t){return void 0===t&&(t=!0),this._rgb.slice(0,4).map((function(r,n){return n<3?!1===t?r:Y(r):r}))},p.rgb=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];return new(Function.prototype.bind.apply(f,[null].concat(t,["rgb"])))},h.format.rgb=function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];var n=u(t,"rgba");return void 0===n[3]&&(n[3]=1),n},h.autodetect.push({p:3,test:function(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];if("array"===i(t=u(t,"rgba"))&&(3===t.length||4===t.length&&"number"==i(t[3])&&t[3]>=0&&t[3]<=1))return"rgb"}}),f.prototype.alpha=function(t,r){return void 0===r&&(r=!1),void 0!==t&&"number"===i(t)?r?(this._rgb[3]=t,this):new f([this._rgb[0],this._rgb[1],this._rgb[2],t],"rgb"):this._rgb[3]},f.prototype.darken=function(t){void 0===t&&(t=1);var r=this.lab();return r[0]-=C*t,new f(r,"lab").alpha(this.alpha(),!0)},f.prototype.brighten=function(t){return void 0===t&&(t=1),this.darken(-t)},f.prototype.darker=f.prototype.darken,f.prototype.brighter=f.prototype.brighten,f.prototype.get=function(t){var r=t.split("."),n=r[0],e=r[1],o=this[n]();if(e){var a=n.indexOf(e)-("ok"===n.substr(0,2)?2:0);if(a>-1)return o[a];throw new Error("unknown channel "+e+" in mode "+n)}return o};var Z={};function tt(t,r,n){void 0===n&&(n=.5);for(var e=[],o=arguments.length-3;o-- >0;)e[o]=arguments[o+3];var a=e[0]||"lrgb";if(Z[a]||e.length||(a=Object.keys(Z)[0]),!Z[a])throw new Error("interpolation mode "+a+" is not defined");return"object"!==i(t)&&(t=new f(t)),"object"!==i(r)&&(r=new f(r)),Z[a](t,r,n).alpha(t.alpha()+n*(r.alpha()-t.alpha()))}f.prototype.mix=f.prototype.interpolate=function(t,r){void 0===r&&(r=.5);for(var n=[],e=arguments.length-2;e-- >0;)n[e]=arguments[e+2];return tt.apply(void 0,[this,t,r].concat(n))},f.prototype.set=function(t,r,n){void 0===n&&(n=!1);var e=t.split("."),o=e[0],a=e[1],u=this[o]();if(a){var l=o.indexOf(a)-("ok"===o.substr(0,2)?2:0);if(l>-1){if("string"==i(r))switch(r.charAt(0)){case"+":case"-":u[l]+=+r;break;case"*":u[l]*=+r.substr(1);break;case"/":u[l]/=+r.substr(1);break;default:u[l]=+r}else{if("number"!==i(r))throw new Error("unsupported value for Color.set");u[l]=r}var s=new f(u,o);return n?(this._rgb=s._rgb,this):s}throw new Error("unknown channel "+a+" in mode "+o)}return u},f.prototype.tint=function(t){void 0===t&&(t=.5);for(var r=[],n=arguments.length-1;n-- >0;)r[n]=arguments[n+1];return tt.apply(void 0,[this,"white",t].concat(r))},f.prototype.shade=function(t){void 0===t&&(t=.5);for(var r=[],n=arguments.length-1;n-- >0;)r[n]=arguments[n+1];return tt.apply(void 0,[this,"black",t].concat(r))};var rt=Math.sqrt,nt=Math.pow;Z.lrgb=function(t,r,n){var e=t._rgb,o=e[0],a=e[1],i=e[2],u=r._rgb,l=u[0],s=u[1],c=u[2];return new f(rt(nt(o,2)*(1-n)+nt(l,2)*n),rt(nt(a,2)*(1-n)+nt(s,2)*n),rt(nt(i,2)*(1-n)+nt(c,2)*n),"rgb")};function et(){for(var t=[],r=arguments.length;r--;)t[r]=arguments[r];try{return new(Function.prototype.bind.apply(f,[null].concat(t))),!0}catch(t){return!1}}Z.oklab=function(t,r,n){var e=t.oklab(),o=r.oklab();return new f(e[0]+n*(o[0]-e[0]),e[1]+n*(o[1]-e[1]),e[2]+n*(o[2]-e[2]),"oklab")},Object.assign(p,{Color:f,valid:et,mix:tt,interpolate:tt}),t.Color=f,t.default=p,t.interpolate=tt,t.mix=tt,t.valid=et,Object.defineProperty(t,"__esModule",{value:!0})}));
