/**
 * chroma.js - JavaScript library for color conversions
 *
 * Copyright (c) 2011-2024, Gregor Aisch
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name Gregor Aisch may not be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL GREGOR AISCH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * -------------------------------------------------------
 *
 * chroma.js includes colors from colorbrewer2.org, which are released under
 * the following license:
 *
 * Copyright (c) 2002 Cynthia Brewer, Mark Harrower,
 * and The Pennsylvania State University.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * ------------------------------------------------------
 *
 * Named colors are taken from X11 Color Names.
 * http://www.w3.org/TR/css3-color/#svg-color
 *
 * @preserve
 */

!function(r,e){"object"==typeof exports&&"undefined"!=typeof module?e(exports):"function"==typeof define&&define.amd?define(["exports"],e):e((r="undefined"!=typeof globalThis?globalThis:r||self).chroma={})}(this,(function(r){"use strict";function e(r,e,t){return void 0===e&&(e=0),void 0===t&&(t=1),h(s(e,r),t)}function t(r){r._clipped=!1,r._unclipped=r.slice(0);for(var t=0;t<=3;t++)t<3?((r[t]<0||r[t]>255)&&(r._clipped=!0),r[t]=e(r[t],0,255)):3===t&&(r[t]=e(r[t],0,1));return r}for(var n={},a=0,f=["Boolean","Number","String","Function","Array","Date","RegExp","Undefined","Null"];a<f.length;a+=1){var o=f[a];n["[object "+o+"]"]=o.toLowerCase()}function u(r){return n[Object.prototype.toString.call(r)]||"object"}function c(r,e){return void 0===e&&(e=null),r.length>=3?Array.prototype.slice.call(r):"object"==u(r[0])&&e?e.split("").filter((function(e){return void 0!==r[0][e]})).map((function(e){return r[0][e]})):r[0]}function i(r){if(r.length<2)return null;var e=r.length-1;return"string"==u(r[e])?r[e].toLowerCase():null}var l=Math.PI,h=Math.min,s=Math.max,d=2*l,b=l/3,g=l/180,p=180/l,v={format:{},autodetect:[]},m=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var n=this;if("object"===u(r[0])&&r[0].constructor&&r[0].constructor===this.constructor)return r[0];var a=i(r),f=!1;if(!a){f=!0,v.sorted||(v.autodetect=v.autodetect.sort((function(r,e){return e.p-r.p})),v.sorted=!0);for(var o=0,c=v.autodetect;o<c.length;o+=1){var l=c[o];if(a=l.test.apply(l,r))break}}if(!v.format[a])throw new Error("unknown format: "+r);var h=v.format[a].apply(null,f?r:r.slice(0,-1));n._rgb=t(h),3===n._rgb.length&&n._rgb.push(1)};m.prototype.toString=function(){return"function"==u(this.hex)?this.hex():"["+this._rgb.join(",")+"]"};var y=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(m,[null].concat(r)))};y.version="2.6.1-0";var w=Math.max;m.prototype.cmyk=function(){return function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var t=c(r,"rgb"),n=t[0],a=t[1],f=t[2],o=1-w(n/=255,w(a/=255,f/=255)),u=o<1?1/(1-o):0;return[(1-n-o)*u,(1-a-o)*u,(1-f-o)*u,o]}(this._rgb)},y.cmyk=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(m,[null].concat(r,["cmyk"])))},v.format.cmyk=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var t=(r=c(r,"cmyk"))[0],n=r[1],a=r[2],f=r[3],o=r.length>4?r[4]:1;return 1===f?[0,0,0,o]:[t>=1?0:255*(1-t)*(1-f),n>=1?0:255*(1-n)*(1-f),a>=1?0:255*(1-a)*(1-f),o]},v.autodetect.push({p:2,test:function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];if("array"===u(r=c(r,"cmyk"))&&4===r.length)return"cmyk"}});var k=function(r){return Math.round(100*r)/100},M=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var t,n,a=(r=c(r,"rgba"))[0],f=r[1],o=r[2],u=h(a/=255,f/=255,o/=255),i=s(a,f,o),l=(i+u)/2;return i===u?(t=0,n=Number.NaN):t=l<.5?(i-u)/(i+u):(i-u)/(2-i-u),a==i?n=(f-o)/(i-u):f==i?n=2+(o-a)/(i-u):o==i&&(n=4+(a-f)/(i-u)),(n*=60)<0&&(n+=360),r.length>3&&void 0!==r[3]?[n,t,l,r[3]]:[n,t,l]},N=Math.round,_=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var t=c(r,"rgba"),n=i(r)||"rgb";return"hsl"==n.substr(0,3)?function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var t=c(r,"hsla"),n=i(r)||"lsa";return t[0]=k(t[0]||0),t[1]=k(100*t[1])+"%",t[2]=k(100*t[2])+"%","hsla"===n||t.length>3&&t[3]<1?(t[3]=t.length>3?t[3]:1,n="hsla"):t.length=3,n+"("+t.join(",")+")"}(M(t),n):(t[0]=N(t[0]),t[1]=N(t[1]),t[2]=N(t[2]),("rgba"===n||t.length>3&&t[3]<1)&&(t[3]=t.length>3?t[3]:1,n="rgba"),n+"("+t.slice(0,"rgb"===n?3:4).join(",")+")")},x=function(){for(var r,e=[],t=arguments.length;t--;)e[t]=arguments[t];var n,a,f,o=(e=c(e,"hsl"))[0],u=e[1],i=e[2];if(0===u)n=a=f=255*i;else{var l=[0,0,0],h=[0,0,0],s=i<.5?i*(1+u):i+u-i*u,d=2*i-s,b=o/360;l[0]=b+1/3,l[1]=b,l[2]=b-1/3;for(var g=0;g<3;g++)l[g]<0&&(l[g]+=1),l[g]>1&&(l[g]-=1),6*l[g]<1?h[g]=d+6*(s-d)*l[g]:2*l[g]<1?h[g]=s:3*l[g]<2?h[g]=d+(s-d)*(2/3-l[g])*6:h[g]=d;n=(r=[255*h[0],255*h[1],255*h[2]])[0],a=r[1],f=r[2]}return e.length>3?[n,a,f,e[3]]:[n,a,f,1]},A=/^rgb\(\s*(-?\d+),\s*(-?\d+)\s*,\s*(-?\d+)\s*\)$/,E=/^rgba\(\s*(-?\d+),\s*(-?\d+)\s*,\s*(-?\d+)\s*,\s*([01]|[01]?\.\d+)\)$/,F=/^rgb\(\s*(-?\d+(?:\.\d+)?)%,\s*(-?\d+(?:\.\d+)?)%\s*,\s*(-?\d+(?:\.\d+)?)%\s*\)$/,j=/^rgba\(\s*(-?\d+(?:\.\d+)?)%,\s*(-?\d+(?:\.\d+)?)%\s*,\s*(-?\d+(?:\.\d+)?)%\s*,\s*([01]|[01]?\.\d+)\)$/,O=/^hsl\(\s*(-?\d+(?:\.\d+)?),\s*(-?\d+(?:\.\d+)?)%\s*,\s*(-?\d+(?:\.\d+)?)%\s*\)$/,q=/^hsla\(\s*(-?\d+(?:\.\d+)?),\s*(-?\d+(?:\.\d+)?)%\s*,\s*(-?\d+(?:\.\d+)?)%\s*,\s*([01]|[01]?\.\d+)\)$/,L=Math.round,P=function(r){var e;if(r=r.toLowerCase().trim(),v.format.named)try{return v.format.named(r)}catch(r){}if(e=r.match(A)){for(var t=e.slice(1,4),n=0;n<3;n++)t[n]=+t[n];return t[3]=1,t}if(e=r.match(E)){for(var a=e.slice(1,5),f=0;f<4;f++)a[f]=+a[f];return a}if(e=r.match(F)){for(var o=e.slice(1,4),u=0;u<3;u++)o[u]=L(2.55*o[u]);return o[3]=1,o}if(e=r.match(j)){for(var c=e.slice(1,5),i=0;i<3;i++)c[i]=L(2.55*c[i]);return c[3]=+c[3],c}if(e=r.match(O)){var l=e.slice(1,4);l[1]*=.01,l[2]*=.01;for(var h=x(l),s=0;s<3;s++)h[s]=L(h[s]);return h[3]=1,h}if(e=r.match(q)){var d=e.slice(1,4);d[1]*=.01,d[2]*=.01;for(var b=x(d),g=0;g<3;g++)b[g]=L(b[g]);return b[3]=+e[4],b}};P.test=function(r){return A.test(r)||E.test(r)||F.test(r)||j.test(r)||O.test(r)||q.test(r)},m.prototype.css=function(r){return _(this._rgb,r)},y.css=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(m,[null].concat(r,["css"])))},v.format.css=P,v.autodetect.push({p:5,test:function(r){for(var e=[],t=arguments.length-1;t-- >0;)e[t]=arguments[t+1];if(!e.length&&"string"===u(r)&&P.test(r))return"css"}}),v.format.gl=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var t=c(r,"rgba");return t[0]*=255,t[1]*=255,t[2]*=255,t},y.gl=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(m,[null].concat(r,["gl"])))},m.prototype.gl=function(){var r=this._rgb;return[r[0]/255,r[1]/255,r[2]/255,r[3]]};var G=Math.floor;m.prototype.hcg=function(){return function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var t,n=c(r,"rgb"),a=n[0],f=n[1],o=n[2],u=h(a,f,o),i=s(a,f,o),l=i-u,d=100*l/255,b=u/(255-l)*100;return 0===l?t=Number.NaN:(a===i&&(t=(f-o)/l),f===i&&(t=2+(o-a)/l),o===i&&(t=4+(a-f)/l),(t*=60)<0&&(t+=360)),[t,d,b]}(this._rgb)},y.hcg=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(m,[null].concat(r,["hcg"])))},v.format.hcg=function(){for(var r,e,t,n,a,f,o=[],u=arguments.length;u--;)o[u]=arguments[u];var i,l,h,s=(o=c(o,"hcg"))[0],d=o[1],b=o[2];b*=255;var g=255*d;if(0===d)i=l=h=b;else{360===s&&(s=0),s>360&&(s-=360),s<0&&(s+=360);var p=G(s/=60),v=s-p,m=b*(1-d),y=m+g*(1-v),w=m+g*v,k=m+g;switch(p){case 0:i=(r=[k,w,m])[0],l=r[1],h=r[2];break;case 1:i=(e=[y,k,m])[0],l=e[1],h=e[2];break;case 2:i=(t=[m,k,w])[0],l=t[1],h=t[2];break;case 3:i=(n=[m,y,k])[0],l=n[1],h=n[2];break;case 4:i=(a=[w,m,k])[0],l=a[1],h=a[2];break;case 5:i=(f=[k,m,y])[0],l=f[1],h=f[2]}}return[i,l,h,o.length>3?o[3]:1]},v.autodetect.push({p:1,test:function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];if("array"===u(r=c(r,"hcg"))&&3===r.length)return"hcg"}});var B=/^#?([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$/,C=/^#?([A-Fa-f0-9]{8}|[A-Fa-f0-9]{4})$/,R=function(r){if(r.match(B)){4!==r.length&&7!==r.length||(r=r.substr(1)),3===r.length&&(r=(r=r.split(""))[0]+r[0]+r[1]+r[1]+r[2]+r[2]);var e=parseInt(r,16);return[e>>16,e>>8&255,255&e,1]}if(r.match(C)){5!==r.length&&9!==r.length||(r=r.substr(1)),4===r.length&&(r=(r=r.split(""))[0]+r[0]+r[1]+r[1]+r[2]+r[2]+r[3]+r[3]);var t=parseInt(r,16);return[t>>24&255,t>>16&255,t>>8&255,Math.round((255&t)/255*100)/100]}throw new Error("unknown hex color: "+r)},S=Math.round,$=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var t=c(r,"rgba"),n=t[0],a=t[1],f=t[2],o=t[3],u=i(r)||"auto";void 0===o&&(o=1),"auto"===u&&(u=o<1?"rgba":"rgb");var l="000000"+((n=S(n))<<16|(a=S(a))<<8|(f=S(f))).toString(16);l=l.substr(l.length-6);var h="0"+S(255*o).toString(16);switch(h=h.substr(h.length-2),u.toLowerCase()){case"rgba":return"#"+l+h;case"argb":return"#"+h+l;default:return"#"+l}};m.prototype.hex=function(r){return $(this._rgb,r)},y.hex=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(m,[null].concat(r,["hex"])))},v.format.hex=R,v.autodetect.push({p:4,test:function(r){for(var e=[],t=arguments.length-1;t-- >0;)e[t]=arguments[t+1];if(!e.length&&"string"===u(r)&&[3,4,5,6,7,8,9].indexOf(r.length)>=0)return"hex"}});var z=Math.cos,Y=Math.min,I=Math.sqrt,U=Math.acos;m.prototype.hsi=function(){return function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var t,n=c(r,"rgb"),a=n[0],f=n[1],o=n[2],u=Y(a/=255,f/=255,o/=255),i=(a+f+o)/3,l=i>0?1-u/i:0;return 0===l?t=NaN:(t=(a-f+(a-o))/2,t/=I((a-f)*(a-f)+(a-o)*(f-o)),t=U(t),o>f&&(t=d-t),t/=d),[360*t,l,i]}(this._rgb)},y.hsi=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(m,[null].concat(r,["hsi"])))},v.format.hsi=function(){for(var r=[],t=arguments.length;t--;)r[t]=arguments[t];var n,a,f,o=(r=c(r,"hsi"))[0],u=r[1],i=r[2];return isNaN(o)&&(o=0),isNaN(u)&&(u=0),o>360&&(o-=360),o<0&&(o+=360),(o/=360)<1/3?a=1-((f=(1-u)/3)+(n=(1+u*z(d*o)/z(b-d*o))/3)):o<2/3?f=1-((n=(1-u)/3)+(a=(1+u*z(d*(o-=1/3))/z(b-d*o))/3)):n=1-((a=(1-u)/3)+(f=(1+u*z(d*(o-=2/3))/z(b-d*o))/3)),[255*(n=e(i*n*3)),255*(a=e(i*a*3)),255*(f=e(i*f*3)),r.length>3?r[3]:1]},v.autodetect.push({p:2,test:function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];if("array"===u(r=c(r,"hsi"))&&3===r.length)return"hsi"}}),m.prototype.hsl=function(){return M(this._rgb)},y.hsl=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(m,[null].concat(r,["hsl"])))},v.format.hsl=x,v.autodetect.push({p:2,test:function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];if("array"===u(r=c(r,"hsl"))&&3===r.length)return"hsl"}});var V=Math.floor,X=Math.min,D=Math.max;m.prototype.hsv=function(){return function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var t,n,a,f=(r=c(r,"rgb"))[0],o=r[1],u=r[2],i=X(f,o,u),l=D(f,o,u),h=l-i;return a=l/255,0===l?(t=Number.NaN,n=0):(n=h/l,f===l&&(t=(o-u)/h),o===l&&(t=2+(u-f)/h),u===l&&(t=4+(f-o)/h),(t*=60)<0&&(t+=360)),[t,n,a]}(this._rgb)},y.hsv=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(m,[null].concat(r,["hsv"])))},v.format.hsv=function(){for(var r,e,t,n,a,f,o=[],u=arguments.length;u--;)o[u]=arguments[u];var i,l,h,s=(o=c(o,"hsv"))[0],d=o[1],b=o[2];if(b*=255,0===d)i=l=h=b;else{360===s&&(s=0),s>360&&(s-=360),s<0&&(s+=360);var g=V(s/=60),p=s-g,v=b*(1-d),m=b*(1-d*p),y=b*(1-d*(1-p));switch(g){case 0:i=(r=[b,y,v])[0],l=r[1],h=r[2];break;case 1:i=(e=[m,b,v])[0],l=e[1],h=e[2];break;case 2:i=(t=[v,b,y])[0],l=t[1],h=t[2];break;case 3:i=(n=[v,m,b])[0],l=n[1],h=n[2];break;case 4:i=(a=[y,v,b])[0],l=a[1],h=a[2];break;case 5:i=(f=[b,v,m])[0],l=f[1],h=f[2]}}return[i,l,h,o.length>3?o[3]:1]},v.autodetect.push({p:2,test:function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];if("array"===u(r=c(r,"hsv"))&&3===r.length)return"hsv"}});var T=18,H=.95047,J=1,K=1.08883,Q=.137931034,W=.206896552,Z=.12841855,rr=.008856452,er=Math.pow,tr=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var t,n,a,f=(r=c(r,"lab"))[0],o=r[1],u=r[2];return n=(f+16)/116,t=isNaN(o)?n:n+o/500,a=isNaN(u)?n:n-u/200,n=J*ar(n),t=H*ar(t),a=K*ar(a),[nr(3.2404542*t-1.5371385*n-.4985314*a),nr(-.969266*t+1.8760108*n+.041556*a),nr(.0556434*t-.2040259*n+1.0572252*a),r.length>3?r[3]:1]},nr=function(r){return 255*(r<=.00304?12.92*r:1.055*er(r,1/2.4)-.055)},ar=function(r){return r>W?r*r*r:Z*(r-Q)},fr=Math.pow,or=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var t=c(r,"rgb"),n=t[0],a=t[1],f=t[2],o=ir(n,a,f),u=o[0],i=o[1],l=116*i-16;return[l<0?0:l,500*(u-i),200*(i-o[2])]},ur=function(r){return(r/=255)<=.04045?r/12.92:fr((r+.055)/1.055,2.4)},cr=function(r){return r>rr?fr(r,1/3):r/Z+Q},ir=function(r,e,t){return r=ur(r),e=ur(e),t=ur(t),[cr((.4124564*r+.3575761*e+.1804375*t)/H),cr((.2126729*r+.7151522*e+.072175*t)/J),cr((.0193339*r+.119192*e+.9503041*t)/K)]};m.prototype.lab=function(){return or(this._rgb)},y.lab=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(m,[null].concat(r,["lab"])))},v.format.lab=tr,v.autodetect.push({p:2,test:function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];if("array"===u(r=c(r,"lab"))&&3===r.length)return"lab"}});var lr=Math.sin,hr=Math.cos,sr=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var t=c(r,"lch"),n=t[0],a=t[1],f=t[2];return isNaN(f)&&(f=0),[n,hr(f*=g)*a,lr(f)*a]},dr=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var t=(r=c(r,"lch"))[0],n=r[1],a=r[2],f=sr(t,n,a),o=f[0],u=f[1],i=f[2],l=tr(o,u,i);return[l[0],l[1],l[2],r.length>3?r[3]:1]},br=Math.sqrt,gr=Math.atan2,pr=Math.round,vr=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var t=c(r,"lab"),n=t[0],a=t[1],f=t[2],o=br(a*a+f*f),u=(gr(f,a)*p+360)%360;return 0===pr(1e4*o)&&(u=Number.NaN),[n,o,u]},mr=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var t=c(r,"rgb"),n=t[0],a=t[1],f=t[2],o=or(n,a,f),u=o[0],i=o[1],l=o[2];return vr(u,i,l)};m.prototype.lch=function(){return mr(this._rgb)},m.prototype.hcl=function(){return mr(this._rgb).reverse()},y.lch=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(m,[null].concat(r,["lch"])))},y.hcl=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(m,[null].concat(r,["hcl"])))},v.format.lch=dr,v.format.hcl=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var t=c(r,"hcl").reverse();return dr.apply(void 0,t)},["lch","hcl"].forEach((function(r){return v.autodetect.push({p:2,test:function(){for(var e=[],t=arguments.length;t--;)e[t]=arguments[t];if("array"===u(e=c(e,r))&&3===e.length)return r}})}));var yr={aliceblue:"#f0f8ff",antiquewhite:"#faebd7",aqua:"#00ffff",aquamarine:"#7fffd4",azure:"#f0ffff",beige:"#f5f5dc",bisque:"#ffe4c4",black:"#000000",blanchedalmond:"#ffebcd",blue:"#0000ff",blueviolet:"#8a2be2",brown:"#a52a2a",burlywood:"#deb887",cadetblue:"#5f9ea0",chartreuse:"#7fff00",chocolate:"#d2691e",coral:"#ff7f50",cornflowerblue:"#6495ed",cornsilk:"#fff8dc",crimson:"#dc143c",cyan:"#00ffff",darkblue:"#00008b",darkcyan:"#008b8b",darkgoldenrod:"#b8860b",darkgray:"#a9a9a9",darkgreen:"#006400",darkgrey:"#a9a9a9",darkkhaki:"#bdb76b",darkmagenta:"#8b008b",darkolivegreen:"#556b2f",darkorange:"#ff8c00",darkorchid:"#9932cc",darkred:"#8b0000",darksalmon:"#e9967a",darkseagreen:"#8fbc8f",darkslateblue:"#483d8b",darkslategray:"#2f4f4f",darkslategrey:"#2f4f4f",darkturquoise:"#00ced1",darkviolet:"#9400d3",deeppink:"#ff1493",deepskyblue:"#00bfff",dimgray:"#696969",dimgrey:"#696969",dodgerblue:"#1e90ff",firebrick:"#b22222",floralwhite:"#fffaf0",forestgreen:"#228b22",fuchsia:"#ff00ff",gainsboro:"#dcdcdc",ghostwhite:"#f8f8ff",gold:"#ffd700",goldenrod:"#daa520",gray:"#808080",green:"#008000",greenyellow:"#adff2f",grey:"#808080",honeydew:"#f0fff0",hotpink:"#ff69b4",indianred:"#cd5c5c",indigo:"#4b0082",ivory:"#fffff0",khaki:"#f0e68c",laserlemon:"#ffff54",lavender:"#e6e6fa",lavenderblush:"#fff0f5",lawngreen:"#7cfc00",lemonchiffon:"#fffacd",lightblue:"#add8e6",lightcoral:"#f08080",lightcyan:"#e0ffff",lightgoldenrod:"#fafad2",lightgoldenrodyellow:"#fafad2",lightgray:"#d3d3d3",lightgreen:"#90ee90",lightgrey:"#d3d3d3",lightpink:"#ffb6c1",lightsalmon:"#ffa07a",lightseagreen:"#20b2aa",lightskyblue:"#87cefa",lightslategray:"#778899",lightslategrey:"#778899",lightsteelblue:"#b0c4de",lightyellow:"#ffffe0",lime:"#00ff00",limegreen:"#32cd32",linen:"#faf0e6",magenta:"#ff00ff",maroon:"#800000",maroon2:"#7f0000",maroon3:"#b03060",mediumaquamarine:"#66cdaa",mediumblue:"#0000cd",mediumorchid:"#ba55d3",mediumpurple:"#9370db",mediumseagreen:"#3cb371",mediumslateblue:"#7b68ee",mediumspringgreen:"#00fa9a",mediumturquoise:"#48d1cc",mediumvioletred:"#c71585",midnightblue:"#191970",mintcream:"#f5fffa",mistyrose:"#ffe4e1",moccasin:"#ffe4b5",navajowhite:"#ffdead",navy:"#000080",oldlace:"#fdf5e6",olive:"#808000",olivedrab:"#6b8e23",orange:"#ffa500",orangered:"#ff4500",orchid:"#da70d6",palegoldenrod:"#eee8aa",palegreen:"#98fb98",paleturquoise:"#afeeee",palevioletred:"#db7093",papayawhip:"#ffefd5",peachpuff:"#ffdab9",peru:"#cd853f",pink:"#ffc0cb",plum:"#dda0dd",powderblue:"#b0e0e6",purple:"#800080",purple2:"#7f007f",purple3:"#a020f0",rebeccapurple:"#663399",red:"#ff0000",rosybrown:"#bc8f8f",royalblue:"#4169e1",saddlebrown:"#8b4513",salmon:"#fa8072",sandybrown:"#f4a460",seagreen:"#2e8b57",seashell:"#fff5ee",sienna:"#a0522d",silver:"#c0c0c0",skyblue:"#87ceeb",slateblue:"#6a5acd",slategray:"#708090",slategrey:"#708090",snow:"#fffafa",springgreen:"#00ff7f",steelblue:"#4682b4",tan:"#d2b48c",teal:"#008080",thistle:"#d8bfd8",tomato:"#ff6347",turquoise:"#40e0d0",violet:"#ee82ee",wheat:"#f5deb3",white:"#ffffff",whitesmoke:"#f5f5f5",yellow:"#ffff00",yellowgreen:"#9acd32"};m.prototype.name=function(){for(var r=$(this._rgb,"rgb"),e=0,t=Object.keys(yr);e<t.length;e+=1){var n=t[e];if(yr[n]===r)return n.toLowerCase()}return r},v.format.named=function(r){if(r=r.toLowerCase(),yr[r])return R(yr[r]);throw new Error("unknown color name: "+r)},v.autodetect.push({p:5,test:function(r){for(var e=[],t=arguments.length-1;t-- >0;)e[t]=arguments[t+1];if(!e.length&&"string"===u(r)&&yr[r.toLowerCase()])return"named"}});m.prototype.num=function(){return function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var t=c(r,"rgb");return(t[0]<<16)+(t[1]<<8)+t[2]}(this._rgb)},y.num=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(m,[null].concat(r,["num"])))},v.format.num=function(r){if("number"==u(r)&&r>=0&&r<=16777215)return[r>>16,r>>8&255,255&r,1];throw new Error("unknown num color: "+r)},v.autodetect.push({p:5,test:function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];if(1===r.length&&"number"===u(r[0])&&r[0]>=0&&r[0]<=16777215)return"num"}});var wr=Math.round;m.prototype.rgb=function(r){return void 0===r&&(r=!0),!1===r?this._rgb.slice(0,3):this._rgb.slice(0,3).map(wr)},m.prototype.rgba=function(r){return void 0===r&&(r=!0),this._rgb.slice(0,4).map((function(e,t){return t<3?!1===r?e:wr(e):e}))},y.rgb=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(m,[null].concat(r,["rgb"])))},v.format.rgb=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var t=c(r,"rgba");return void 0===t[3]&&(t[3]=1),t},v.autodetect.push({p:3,test:function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];if("array"===u(r=c(r,"rgba"))&&(3===r.length||4===r.length&&"number"==u(r[3])&&r[3]>=0&&r[3]<=1))return"rgb"}});var kr=Math.log,Mr=function(r){var e,t,n,a=r/100;return a<66?(e=255,t=a<6?0:-155.25485562709179-.44596950469579133*(t=a-2)+104.49216199393888*kr(t),n=a<20?0:.8274096064007395*(n=a-10)-254.76935184120902+115.67994401066147*kr(n)):(e=351.97690566805693+.114206453784165*(e=a-55)-40.25366309332127*kr(e),t=325.4494125711974+.07943456536662342*(t=a-50)-28.0852963507957*kr(t),n=255),[e,t,n,1]},Nr=Math.round;m.prototype.temp=m.prototype.kelvin=m.prototype.temperature=function(){return function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];for(var t,n=c(r,"rgb"),a=n[0],f=n[2],o=1e3,u=4e4;u-o>.4;){var i=Mr(t=.5*(u+o));i[2]/i[0]>=f/a?u=t:o=t}return Nr(t)}(this._rgb)},y.temp=y.kelvin=y.temperature=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(m,[null].concat(r,["temp"])))},v.format.temp=v.format.kelvin=v.format.temperature=Mr;var _r=Math.pow,xr=Math.sign,Ar=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var t=(r=c(r,"lab"))[0],n=r[1],a=r[2],f=_r(t+.3963377774*n+.2158037573*a,3),o=_r(t-.1055613458*n-.0638541728*a,3),u=_r(t-.0894841775*n-1.291485548*a,3);return[255*Er(4.0767416621*f-3.3077115913*o+.2309699292*u),255*Er(-1.2684380046*f+2.6097574011*o-.3413193965*u),255*Er(-.0041960863*f-.7034186147*o+1.707614701*u),r.length>3?r[3]:1]};function Er(r){var e=Math.abs(r);return e>.0031308?(xr(r)||1)*(1.055*_r(e,1/2.4)-.055):12.92*r}var Fr=Math.cbrt,jr=Math.pow,Or=Math.sign,qr=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var t=c(r,"rgb"),n=t[0],a=t[1],f=t[2],o=[Lr(n/255),Lr(a/255),Lr(f/255)],u=o[0],i=o[1],l=o[2],h=Fr(.4122214708*u+.5363325363*i+.0514459929*l),s=Fr(.2119034982*u+.6806995451*i+.1073969566*l),d=Fr(.0883024619*u+.2817188376*i+.6299787005*l);return[.2104542553*h+.793617785*s-.0040720468*d,1.9779984951*h-2.428592205*s+.4505937099*d,.0259040371*h+.7827717662*s-.808675766*d]};function Lr(r){var e=Math.abs(r);return e<.04045?r/12.92:(Or(r)||1)*jr((e+.055)/1.055,2.4)}m.prototype.oklab=function(){return qr(this._rgb)},y.oklab=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(m,[null].concat(r,["oklab"])))},v.format.oklab=Ar,v.autodetect.push({p:3,test:function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];if("array"===u(r=c(r,"oklab"))&&3===r.length)return"oklab"}});m.prototype.oklch=function(){return function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var t=c(r,"rgb"),n=t[0],a=t[1],f=t[2],o=qr(n,a,f),u=o[0],i=o[1],l=o[2];return vr(u,i,l)}(this._rgb)},y.oklch=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];return new(Function.prototype.bind.apply(m,[null].concat(r,["oklch"])))},v.format.oklch=function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];var t=(r=c(r,"lch"))[0],n=r[1],a=r[2],f=sr(t,n,a),o=f[0],u=f[1],i=f[2],l=Ar(o,u,i);return[l[0],l[1],l[2],r.length>3?r[3]:1]},v.autodetect.push({p:3,test:function(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];if("array"===u(r=c(r,"oklch"))&&3===r.length)return"oklch"}}),m.prototype.alpha=function(r,e){return void 0===e&&(e=!1),void 0!==r&&"number"===u(r)?e?(this._rgb[3]=r,this):new m([this._rgb[0],this._rgb[1],this._rgb[2],r],"rgb"):this._rgb[3]},m.prototype.clipped=function(){return this._rgb._clipped||!1},m.prototype.darken=function(r){void 0===r&&(r=1);var e=this.lab();return e[0]-=T*r,new m(e,"lab").alpha(this.alpha(),!0)},m.prototype.brighten=function(r){return void 0===r&&(r=1),this.darken(-r)},m.prototype.darker=m.prototype.darken,m.prototype.brighter=m.prototype.brighten,m.prototype.get=function(r){var e=r.split("."),t=e[0],n=e[1],a=this[t]();if(n){var f=t.indexOf(n)-("ok"===t.substr(0,2)?2:0);if(f>-1)return a[f];throw new Error("unknown channel "+n+" in mode "+t)}return a};var Pr=Math.pow;m.prototype.luminance=function(r,e){if(void 0===e&&(e="rgb"),void 0!==r&&"number"===u(r)){if(0===r)return new m([0,0,0,this._rgb[3]],"rgb");if(1===r)return new m([255,255,255,this._rgb[3]],"rgb");var t=this.luminance(),n=20,a=function(t,f){var o=t.interpolate(f,.5,e),u=o.luminance();return Math.abs(r-u)<1e-7||!n--?o:u>r?a(t,o):a(o,f)},f=(t>r?a(new m([0,0,0]),this):a(this,new m([255,255,255]))).rgb();return new m(f.concat([this._rgb[3]]))}return Gr.apply(void 0,this._rgb.slice(0,3))};var Gr=function(r,e,t){return.2126*(r=Br(r))+.7152*(e=Br(e))+.0722*(t=Br(t))},Br=function(r){return(r/=255)<=.03928?r/12.92:Pr((r+.055)/1.055,2.4)},Cr={};function Rr(r,e,t){void 0===t&&(t=.5);for(var n=[],a=arguments.length-3;a-- >0;)n[a]=arguments[a+3];var f=n[0]||"lrgb";if(Cr[f]||n.length||(f=Object.keys(Cr)[0]),!Cr[f])throw new Error("interpolation mode "+f+" is not defined");return"object"!==u(r)&&(r=new m(r)),"object"!==u(e)&&(e=new m(e)),Cr[f](r,e,t).alpha(r.alpha()+t*(e.alpha()-r.alpha()))}m.prototype.mix=m.prototype.interpolate=function(r,e){void 0===e&&(e=.5);for(var t=[],n=arguments.length-2;n-- >0;)t[n]=arguments[n+2];return Rr.apply(void 0,[this,r,e].concat(t))},m.prototype.premultiply=function(r){void 0===r&&(r=!1);var e=this._rgb,t=e[3];return r?(this._rgb=[e[0]*t,e[1]*t,e[2]*t,t],this):new m([e[0]*t,e[1]*t,e[2]*t,t],"rgb")},m.prototype.saturate=function(r){void 0===r&&(r=1);var e=this.lch();return e[1]+=T*r,e[1]<0&&(e[1]=0),new m(e,"lch").alpha(this.alpha(),!0)},m.prototype.desaturate=function(r){return void 0===r&&(r=1),this.saturate(-r)},m.prototype.set=function(r,e,t){void 0===t&&(t=!1);var n=r.split("."),a=n[0],f=n[1],o=this[a]();if(f){var c=a.indexOf(f)-("ok"===a.substr(0,2)?2:0);if(c>-1){if("string"==u(e))switch(e.charAt(0)){case"+":case"-":o[c]+=+e;break;case"*":o[c]*=+e.substr(1);break;case"/":o[c]/=+e.substr(1);break;default:o[c]=+e}else{if("number"!==u(e))throw new Error("unsupported value for Color.set");o[c]=e}var i=new m(o,a);return t?(this._rgb=i._rgb,this):i}throw new Error("unknown channel "+f+" in mode "+a)}return o},m.prototype.tint=function(r){void 0===r&&(r=.5);for(var e=[],t=arguments.length-1;t-- >0;)e[t]=arguments[t+1];return Rr.apply(void 0,[this,"white",r].concat(e))},m.prototype.shade=function(r){void 0===r&&(r=.5);for(var e=[],t=arguments.length-1;t-- >0;)e[t]=arguments[t+1];return Rr.apply(void 0,[this,"black",r].concat(e))};Cr.rgb=function(r,e,t){var n=r._rgb,a=e._rgb;return new m(n[0]+t*(a[0]-n[0]),n[1]+t*(a[1]-n[1]),n[2]+t*(a[2]-n[2]),"rgb")};var Sr=Math.sqrt,$r=Math.pow;Cr.lrgb=function(r,e,t){var n=r._rgb,a=n[0],f=n[1],o=n[2],u=e._rgb,c=u[0],i=u[1],l=u[2];return new m(Sr($r(a,2)*(1-t)+$r(c,2)*t),Sr($r(f,2)*(1-t)+$r(i,2)*t),Sr($r(o,2)*(1-t)+$r(l,2)*t),"rgb")};function zr(r,e,t,n){var a,f,o,u,c,i,l,h,s,d,b,g,p;return"hsl"===n?(o=r.hsl(),u=e.hsl()):"hsv"===n?(o=r.hsv(),u=e.hsv()):"hcg"===n?(o=r.hcg(),u=e.hcg()):"hsi"===n?(o=r.hsi(),u=e.hsi()):"lch"===n||"hcl"===n?(n="hcl",o=r.hcl(),u=e.hcl()):"oklch"===n&&(o=r.oklch().reverse(),u=e.oklch().reverse()),"h"!==n.substr(0,1)&&"oklch"!==n||(c=(a=o)[0],l=a[1],s=a[2],i=(f=u)[0],h=f[1],d=f[2]),isNaN(c)||isNaN(i)?isNaN(c)?isNaN(i)?g=Number.NaN:(g=i,1!=s&&0!=s||"hsv"==n||(b=h)):(g=c,1!=d&&0!=d||"hsv"==n||(b=l)):g=c+t*(i>c&&i-c>180?i-(c+360):i<c&&c-i>180?i+360-c:i-c),void 0===b&&(b=l+t*(h-l)),p=s+t*(d-s),new m("oklch"===n?[p,b,g]:[g,b,p],n)}Cr.lab=function(r,e,t){var n=r.lab(),a=e.lab();return new m(n[0]+t*(a[0]-n[0]),n[1]+t*(a[1]-n[1]),n[2]+t*(a[2]-n[2]),"lab")};var Yr=function(r,e,t){return zr(r,e,t,"lch")};Cr.lch=Yr,Cr.hcl=Yr;Cr.num=function(r,e,t){var n=r.num(),a=e.num();return new m(n+t*(a-n),"num")};Cr.hcg=function(r,e,t){return zr(r,e,t,"hcg")};Cr.hsi=function(r,e,t){return zr(r,e,t,"hsi")};Cr.hsl=function(r,e,t){return zr(r,e,t,"hsl")};Cr.hsv=function(r,e,t){return zr(r,e,t,"hsv")};Cr.oklab=function(r,e,t){var n=r.oklab(),a=e.oklab();return new m(n[0]+t*(a[0]-n[0]),n[1]+t*(a[1]-n[1]),n[2]+t*(a[2]-n[2]),"oklab")};Cr.oklch=function(r,e,t){return zr(r,e,t,"oklch")};var Ir=Math.pow,Ur=Math.sqrt,Vr=Math.PI,Xr=Math.cos,Dr=Math.sin,Tr=Math.atan2;function Hr(r,e,t){void 0===e&&(e="lrgb"),void 0===t&&(t=null);var n=r.length;t||(t=Array.from(new Array(n)).map((function(){return 1})));var a=n/t.reduce((function(r,e){return r+e}));if(t.forEach((function(r,e){t[e]*=a})),r=r.map((function(r){return new m(r)})),"lrgb"===e)return Jr(r,t);for(var f=r.shift(),o=f.get(e),u=[],c=0,i=0,l=0;l<o.length;l++)if(o[l]=(o[l]||0)*t[0],u.push(isNaN(o[l])?0:t[0]),"h"===e.charAt(l)&&!isNaN(o[l])){var h=o[l]/180*Vr;c+=Xr(h)*t[0],i+=Dr(h)*t[0]}var s=f.alpha()*t[0];r.forEach((function(r,n){var a=r.get(e);s+=r.alpha()*t[n+1];for(var f=0;f<o.length;f++)if(!isNaN(a[f]))if(u[f]+=t[n+1],"h"===e.charAt(f)){var l=a[f]/180*Vr;c+=Xr(l)*t[n+1],i+=Dr(l)*t[n+1]}else o[f]+=a[f]*t[n+1]}));for(var d=0;d<o.length;d++)if("h"===e.charAt(d)){for(var b=Tr(i/u[d],c/u[d])/Vr*180;b<0;)b+=360;for(;b>=360;)b-=360;o[d]=b}else o[d]=o[d]/u[d];return s/=n,new m(o,e).alpha(s>.99999?1:s,!0)}var Jr=function(r,e){for(var n=r.length,a=[0,0,0,0],f=0;f<r.length;f++){var o=r[f],u=e[f]/n,c=o._rgb;a[0]+=Ir(c[0],2)*u,a[1]+=Ir(c[1],2)*u,a[2]+=Ir(c[2],2)*u,a[3]+=c[3]*u}return a[0]=Ur(a[0]),a[1]=Ur(a[1]),a[2]=Ur(a[2]),a[3]>.9999999&&(a[3]=1),new m(t(a))},Kr=Math.pow;function Qr(r){var t="rgb",n=y("#ccc"),a=0,f=[0,1],o=[],c=[0,0],i=!1,l=[],h=!1,s=0,d=1,b=!1,g={},p=!0,v=1,m=function(r){if((r=r||["#fff","#000"])&&"string"===u(r)&&y.brewer&&y.brewer[r.toLowerCase()]&&(r=y.brewer[r.toLowerCase()]),"array"===u(r)){1===r.length&&(r=[r[0],r[0]]),r=r.slice(0);for(var e=0;e<r.length;e++)r[e]=y(r[e]);o.length=0;for(var t=0;t<r.length;t++)o.push(t/(r.length-1))}return N(),l=r},w=function(r){return r},k=function(r){return r},M=function(r,a){var f,h;if(null==a&&(a=!1),isNaN(r)||null===r)return n;if(a)h=r;else if(i&&i.length>2){var b=function(r){if(null!=i){for(var e=i.length-1,t=0;t<e&&r>=i[t];)t++;return t-1}return 0}(r);h=b/(i.length-2)}else h=d!==s?(r-s)/(d-s):1;h=k(h),a||(h=w(h)),1!==v&&(h=Kr(h,v)),h=e(h=c[0]+h*(1-c[0]-c[1]),0,1);var m=Math.floor(1e4*h);if(p&&g[m])f=g[m];else{if("array"===u(l))for(var M=0;M<o.length;M++){var N=o[M];if(h<=N){f=l[M];break}if(h>=N&&M===o.length-1){f=l[M];break}if(h>N&&h<o[M+1]){h=(h-N)/(o[M+1]-N),f=y.interpolate(l[M],l[M+1],h,t);break}}else"function"===u(l)&&(f=l(h));p&&(g[m]=f)}return f},N=function(){return g={}};m(r);var _=function(r){var e=y(M(r));return h&&e[h]?e[h]():e};return _.classes=function(r){if(null!=r){if("array"===u(r))i=r,f=[r[0],r[r.length-1]];else{var e=y.analyze(f);i=0===r?[e.min,e.max]:y.limits(e,"e",r)}return _}return i},_.domain=function(r){if(!arguments.length)return f;s=r[0],d=r[r.length-1],o=[];var e=l.length;if(r.length===e&&s!==d)for(var t=0,n=Array.from(r);t<n.length;t+=1){var a=n[t];o.push((a-s)/(d-s))}else{for(var u=0;u<e;u++)o.push(u/(e-1));if(r.length>2){var c=r.map((function(e,t){return t/(r.length-1)})),i=r.map((function(r){return(r-s)/(d-s)}));i.every((function(r,e){return c[e]===r}))||(k=function(r){if(r<=0||r>=1)return r;for(var e=0;r>=i[e+1];)e++;var t=(r-i[e])/(i[e+1]-i[e]);return c[e]+t*(c[e+1]-c[e])})}}return f=[s,d],_},_.mode=function(r){return arguments.length?(t=r,N(),_):t},_.range=function(r,e){return m(r),_},_.out=function(r){return h=r,_},_.spread=function(r){return arguments.length?(a=r,_):a},_.correctLightness=function(r){return null==r&&(r=!0),b=r,N(),w=b?function(r){for(var e=M(0,!0).lab()[0],t=M(1,!0).lab()[0],n=e>t,a=M(r,!0).lab()[0],f=e+(t-e)*r,o=a-f,u=0,c=1,i=20;Math.abs(o)>.01&&i-- >0;)n&&(o*=-1),o<0?(u=r,r+=.5*(c-r)):(c=r,r+=.5*(u-r)),a=M(r,!0).lab()[0],o=a-f;return r}:function(r){return r},_},_.padding=function(r){return null!=r?("number"===u(r)&&(r=[r,r]),c=r,_):c},_.colors=function(e,t){arguments.length<2&&(t="hex");var n=[];if(0===arguments.length)n=l.slice(0);else if(1===e)n=[_(.5)];else if(e>1){var a=f[0],o=f[1]-a;n=function(r,e){for(var t=[],n=r<e,a=e,f=r;n?f<a:f>a;n?f++:f--)t.push(f);return t}(0,e).map((function(r){return _(a+r/(e-1)*o)}))}else{r=[];var u=[];if(i&&i.length>2)for(var c=1,h=i.length,s=1<=h;s?c<h:c>h;s?c++:c--)u.push(.5*(i[c-1]+i[c]));else u=f;n=u.map((function(r){return _(r)}))}return y[t]&&(n=n.map((function(r){return r[t]()}))),n},_.cache=function(r){return null!=r?(p=r,_):p},_.gamma=function(r){return null!=r?(v=r,_):v},_.nodata=function(r){return null!=r?(n=y(r),_):n},_}function Wr(r){var e=function(r){var e,t,n,a,f,o,u;if(2===(r=r.map((function(r){return new m(r)}))).length)e=r.map((function(r){return r.lab()})),f=e[0],o=e[1],a=function(r){var e=[0,1,2].map((function(e){return f[e]+r*(o[e]-f[e])}));return new m(e,"lab")};else if(3===r.length)t=r.map((function(r){return r.lab()})),f=t[0],o=t[1],u=t[2],a=function(r){var e=[0,1,2].map((function(e){return(1-r)*(1-r)*f[e]+2*(1-r)*r*o[e]+r*r*u[e]}));return new m(e,"lab")};else if(4===r.length){var c;n=r.map((function(r){return r.lab()})),f=n[0],o=n[1],u=n[2],c=n[3],a=function(r){var e=[0,1,2].map((function(e){return(1-r)*(1-r)*(1-r)*f[e]+3*(1-r)*(1-r)*r*o[e]+3*(1-r)*r*r*u[e]+r*r*r*c[e]}));return new m(e,"lab")}}else{if(!(r.length>=5))throw new RangeError("No point in running bezier with only one color.");var i,l,h;i=r.map((function(r){return r.lab()})),h=r.length-1,l=function(r){for(var e=[1,1],t=1;t<r;t++){for(var n=[1],a=1;a<=e.length;a++)n[a]=(e[a]||0)+e[a-1];e=n}return e}(h),a=function(r){var e=1-r,t=[0,1,2].map((function(t){return i.reduce((function(n,a,f){return n+l[f]*Math.pow(e,h-f)*Math.pow(r,f)*a[t]}),0)}));return new m(t,"lab")}}return a}(r);return e.scale=function(){return Qr(e)},e}var Zr=function(r,e,t){if(!Zr[t])throw new Error("unknown blend mode "+t);return Zr[t](r,e)},re=function(r){return function(e,t){var n=y(t).rgb(),a=y(e).rgb();return y.rgb(r(n,a))}},ee=function(r){return function(e,t){var n=[];return n[0]=r(e[0],t[0]),n[1]=r(e[1],t[1]),n[2]=r(e[2],t[2]),n}};Zr.normal=re(ee((function(r){return r}))),Zr.multiply=re(ee((function(r,e){return r*e/255}))),Zr.screen=re(ee((function(r,e){return 255*(1-(1-r/255)*(1-e/255))}))),Zr.overlay=re(ee((function(r,e){return e<128?2*r*e/255:255*(1-2*(1-r/255)*(1-e/255))}))),Zr.darken=re(ee((function(r,e){return r>e?e:r}))),Zr.lighten=re(ee((function(r,e){return r>e?r:e}))),Zr.dodge=re(ee((function(r,e){return 255===r||(r=e/255*255/(1-r/255))>255?255:r}))),Zr.burn=re(ee((function(r,e){return 255*(1-(1-e/255)/(r/255))})));var te=Math.pow,ne=Math.sin,ae=Math.cos;function fe(r,e,n,a,f){void 0===r&&(r=300),void 0===e&&(e=-1.5),void 0===n&&(n=1),void 0===a&&(a=1),void 0===f&&(f=[0,1]);var o,c=0;"array"===u(f)?o=f[1]-f[0]:(o=0,f=[f,f]);var i=function(u){var i=d*((r+120)/360+e*u),l=te(f[0]+o*u,a),h=(0!==c?n[0]+u*c:n)*l*(1-l)/2,s=ae(i),b=ne(i);return y(t([255*(l+h*(-.14861*s+1.78277*b)),255*(l+h*(-.29227*s-.90649*b)),255*(l+h*(1.97294*s)),1]))};return i.start=function(e){return null==e?r:(r=e,i)},i.rotations=function(r){return null==r?e:(e=r,i)},i.gamma=function(r){return null==r?a:(a=r,i)},i.hue=function(r){return null==r?n:("array"===u(n=r)?0===(c=n[1]-n[0])&&(n=n[1]):c=0,i)},i.lightness=function(r){return null==r?f:("array"===u(r)?(f=r,o=r[1]-r[0]):(f=[r,r],o=0),i)},i.scale=function(){return y.scale(i)},i.hue(n),i}var oe=Math.floor,ue=Math.random;function ce(){for(var r="#",e=0;e<6;e++)r+="0123456789abcdef".charAt(oe(16*ue()));return new m(r,"hex")}var ie=Math.log,le=Math.pow,he=Math.floor,se=Math.abs;function de(r,e){void 0===e&&(e=null);var t={min:Number.MAX_VALUE,max:-1*Number.MAX_VALUE,sum:0,values:[],count:0};return"object"===u(r)&&(r=Object.values(r)),r.forEach((function(r){e&&"object"===u(r)&&(r=r[e]),null==r||isNaN(r)||(t.values.push(r),t.sum+=r,r<t.min&&(t.min=r),r>t.max&&(t.max=r),t.count+=1)})),t.domain=[t.min,t.max],t.limits=function(r,e){return be(t,r,e)},t}function be(r,e,t){void 0===e&&(e="equal"),void 0===t&&(t=7),"array"==u(r)&&(r=de(r));var n=r.min,a=r.max,f=r.values.sort((function(r,e){return r-e}));if(1===t)return[n,a];var o=[];if("c"===e.substr(0,1)&&(o.push(n),o.push(a)),"e"===e.substr(0,1)){o.push(n);for(var c=1;c<t;c++)o.push(n+c/t*(a-n));o.push(a)}else if("l"===e.substr(0,1)){if(n<=0)throw new Error("Logarithmic scales are only possible for values > 0");var i=Math.LOG10E*ie(n),l=Math.LOG10E*ie(a);o.push(n);for(var h=1;h<t;h++)o.push(le(10,i+h/t*(l-i)));o.push(a)}else if("q"===e.substr(0,1)){o.push(n);for(var s=1;s<t;s++){var d=(f.length-1)*s/t,b=he(d);if(b===d)o.push(f[b]);else{var g=d-b;o.push(f[b]*(1-g)+f[b+1]*g)}}o.push(a)}else if("k"===e.substr(0,1)){var p,v=f.length,m=new Array(v),y=new Array(t),w=!0,k=0,M=null;(M=[]).push(n);for(var N=1;N<t;N++)M.push(n+N/t*(a-n));for(M.push(a);w;){for(var _=0;_<t;_++)y[_]=0;for(var x=0;x<v;x++)for(var A=f[x],E=Number.MAX_VALUE,F=void 0,j=0;j<t;j++){var O=se(M[j]-A);O<E&&(E=O,F=j),y[F]++,m[x]=F}for(var q=new Array(t),L=0;L<t;L++)q[L]=null;for(var P=0;P<v;P++)null===q[p=m[P]]?q[p]=f[P]:q[p]+=f[P];for(var G=0;G<t;G++)q[G]*=1/y[G];w=!1;for(var B=0;B<t;B++)if(q[B]!==M[B]){w=!0;break}M=q,++k>200&&(w=!1)}for(var C={},R=0;R<t;R++)C[R]=[];for(var S=0;S<v;S++)C[p=m[S]].push(f[S]);for(var $=[],z=0;z<t;z++)$.push(C[z][0]),$.push(C[z][C[z].length-1]);$=$.sort((function(r,e){return r-e})),o.push($[0]);for(var Y=1;Y<$.length;Y+=2){var I=$[Y];isNaN(I)||-1!==o.indexOf(I)||o.push(I)}}return o}function ge(r,e){r=new m(r),e=new m(e);var t=r.luminance(),n=e.luminance();return t>n?(t+.05)/(n+.05):(n+.05)/(t+.05)}var pe=Math.sqrt,ve=Math.pow,me=Math.min,ye=Math.max,we=Math.atan2,ke=Math.abs,Me=Math.cos,Ne=Math.sin,_e=Math.exp,xe=Math.PI;function Ae(r,e,t,n,a){void 0===t&&(t=1),void 0===n&&(n=1),void 0===a&&(a=1);var f=function(r){return 360*r/(2*xe)},o=function(r){return 2*xe*r/360};r=new m(r),e=new m(e);var u=Array.from(r.lab()),c=u[0],i=u[1],l=u[2],h=Array.from(e.lab()),s=h[0],d=h[1],b=h[2],g=(c+s)/2,p=(pe(ve(i,2)+ve(l,2))+pe(ve(d,2)+ve(b,2)))/2,v=.5*(1-pe(ve(p,7)/(ve(p,7)+ve(25,7)))),y=i*(1+v),w=d*(1+v),k=pe(ve(y,2)+ve(l,2)),M=pe(ve(w,2)+ve(b,2)),N=(k+M)/2,_=f(we(l,y)),x=f(we(b,w)),A=_>=0?_:_+360,E=x>=0?x:x+360,F=ke(A-E)>180?(A+E+360)/2:(A+E)/2,j=1-.17*Me(o(F-30))+.24*Me(o(2*F))+.32*Me(o(3*F+6))-.2*Me(o(4*F-63)),O=E-A;O=ke(O)<=180?O:E<=A?O+360:O-360,O=2*pe(k*M)*Ne(o(O)/2);var q=s-c,L=M-k,P=1+.015*ve(g-50,2)/pe(20+ve(g-50,2)),G=1+.045*N,B=1+.015*N*j,C=30*_e(-ve((F-275)/25,2)),R=-(2*pe(ve(N,7)/(ve(N,7)+ve(25,7))))*Ne(2*o(C)),S=pe(ve(q/(t*P),2)+ve(L/(n*G),2)+ve(O/(a*B),2)+R*(L/(n*G))*(O/(a*B)));return ye(0,me(100,S))}function Ee(r,e,t){void 0===t&&(t="lab"),r=new m(r),e=new m(e);var n=r.get(t),a=e.get(t),f=0;for(var o in n){var u=(n[o]||0)-(a[o]||0);f+=u*u}return Math.sqrt(f)}function Fe(){for(var r=[],e=arguments.length;e--;)r[e]=arguments[e];try{return new(Function.prototype.bind.apply(m,[null].concat(r))),!0}catch(r){return!1}}for(var je={cool:function(){return Qr([y.hsl(180,1,.9),y.hsl(250,.7,.4)])},hot:function(){return Qr(["#000","#f00","#ff0","#fff"]).mode("rgb")}},Oe={OrRd:["#fff7ec","#fee8c8","#fdd49e","#fdbb84","#fc8d59","#ef6548","#d7301f","#b30000","#7f0000"],PuBu:["#fff7fb","#ece7f2","#d0d1e6","#a6bddb","#74a9cf","#3690c0","#0570b0","#045a8d","#023858"],BuPu:["#f7fcfd","#e0ecf4","#bfd3e6","#9ebcda","#8c96c6","#8c6bb1","#88419d","#810f7c","#4d004b"],Oranges:["#fff5eb","#fee6ce","#fdd0a2","#fdae6b","#fd8d3c","#f16913","#d94801","#a63603","#7f2704"],BuGn:["#f7fcfd","#e5f5f9","#ccece6","#99d8c9","#66c2a4","#41ae76","#238b45","#006d2c","#00441b"],YlOrBr:["#ffffe5","#fff7bc","#fee391","#fec44f","#fe9929","#ec7014","#cc4c02","#993404","#662506"],YlGn:["#ffffe5","#f7fcb9","#d9f0a3","#addd8e","#78c679","#41ab5d","#238443","#006837","#004529"],Reds:["#fff5f0","#fee0d2","#fcbba1","#fc9272","#fb6a4a","#ef3b2c","#cb181d","#a50f15","#67000d"],RdPu:["#fff7f3","#fde0dd","#fcc5c0","#fa9fb5","#f768a1","#dd3497","#ae017e","#7a0177","#49006a"],Greens:["#f7fcf5","#e5f5e0","#c7e9c0","#a1d99b","#74c476","#41ab5d","#238b45","#006d2c","#00441b"],YlGnBu:["#ffffd9","#edf8b1","#c7e9b4","#7fcdbb","#41b6c4","#1d91c0","#225ea8","#253494","#081d58"],Purples:["#fcfbfd","#efedf5","#dadaeb","#bcbddc","#9e9ac8","#807dba","#6a51a3","#54278f","#3f007d"],GnBu:["#f7fcf0","#e0f3db","#ccebc5","#a8ddb5","#7bccc4","#4eb3d3","#2b8cbe","#0868ac","#084081"],Greys:["#ffffff","#f0f0f0","#d9d9d9","#bdbdbd","#969696","#737373","#525252","#252525","#000000"],YlOrRd:["#ffffcc","#ffeda0","#fed976","#feb24c","#fd8d3c","#fc4e2a","#e31a1c","#bd0026","#800026"],PuRd:["#f7f4f9","#e7e1ef","#d4b9da","#c994c7","#df65b0","#e7298a","#ce1256","#980043","#67001f"],Blues:["#f7fbff","#deebf7","#c6dbef","#9ecae1","#6baed6","#4292c6","#2171b5","#08519c","#08306b"],PuBuGn:["#fff7fb","#ece2f0","#d0d1e6","#a6bddb","#67a9cf","#3690c0","#02818a","#016c59","#014636"],Viridis:["#440154","#482777","#3f4a8a","#31678e","#26838f","#1f9d8a","#6cce5a","#b6de2b","#fee825"],Spectral:["#9e0142","#d53e4f","#f46d43","#fdae61","#fee08b","#ffffbf","#e6f598","#abdda4","#66c2a5","#3288bd","#5e4fa2"],RdYlGn:["#a50026","#d73027","#f46d43","#fdae61","#fee08b","#ffffbf","#d9ef8b","#a6d96a","#66bd63","#1a9850","#006837"],RdBu:["#67001f","#b2182b","#d6604d","#f4a582","#fddbc7","#f7f7f7","#d1e5f0","#92c5de","#4393c3","#2166ac","#053061"],PiYG:["#8e0152","#c51b7d","#de77ae","#f1b6da","#fde0ef","#f7f7f7","#e6f5d0","#b8e186","#7fbc41","#4d9221","#276419"],PRGn:["#40004b","#762a83","#9970ab","#c2a5cf","#e7d4e8","#f7f7f7","#d9f0d3","#a6dba0","#5aae61","#1b7837","#00441b"],RdYlBu:["#a50026","#d73027","#f46d43","#fdae61","#fee090","#ffffbf","#e0f3f8","#abd9e9","#74add1","#4575b4","#313695"],BrBG:["#543005","#8c510a","#bf812d","#dfc27d","#f6e8c3","#f5f5f5","#c7eae5","#80cdc1","#35978f","#01665e","#003c30"],RdGy:["#67001f","#b2182b","#d6604d","#f4a582","#fddbc7","#ffffff","#e0e0e0","#bababa","#878787","#4d4d4d","#1a1a1a"],PuOr:["#7f3b08","#b35806","#e08214","#fdb863","#fee0b6","#f7f7f7","#d8daeb","#b2abd2","#8073ac","#542788","#2d004b"],Set2:["#66c2a5","#fc8d62","#8da0cb","#e78ac3","#a6d854","#ffd92f","#e5c494","#b3b3b3"],Accent:["#7fc97f","#beaed4","#fdc086","#ffff99","#386cb0","#f0027f","#bf5b17","#666666"],Set1:["#e41a1c","#377eb8","#4daf4a","#984ea3","#ff7f00","#ffff33","#a65628","#f781bf","#999999"],Set3:["#8dd3c7","#ffffb3","#bebada","#fb8072","#80b1d3","#fdb462","#b3de69","#fccde5","#d9d9d9","#bc80bd","#ccebc5","#ffed6f"],Dark2:["#1b9e77","#d95f02","#7570b3","#e7298a","#66a61e","#e6ab02","#a6761d","#666666"],Paired:["#a6cee3","#1f78b4","#b2df8a","#33a02c","#fb9a99","#e31a1c","#fdbf6f","#ff7f00","#cab2d6","#6a3d9a","#ffff99","#b15928"],Pastel2:["#b3e2cd","#fdcdac","#cbd5e8","#f4cae4","#e6f5c9","#fff2ae","#f1e2cc","#cccccc"],Pastel1:["#fbb4ae","#b3cde3","#ccebc5","#decbe4","#fed9a6","#ffffcc","#e5d8bd","#fddaec","#f2f2f2"]},qe=0,Le=Object.keys(Oe);qe<Le.length;qe+=1){var Pe=Le[qe];Oe[Pe.toLowerCase()]=Oe[Pe]}Object.assign(y,{analyze:de,average:Hr,bezier:Wr,blend:Zr,brewer:Oe,Color:m,colors:yr,contrast:ge,cubehelix:fe,deltaE:Ae,distance:Ee,input:v,interpolate:Rr,limits:be,mix:Rr,random:ce,scale:Qr,scales:je,valid:Fe}),r.Color=m,r.analyze=de,r.average=Hr,r.bezier=Wr,r.blend=Zr,r.brewer=Oe,r.colors=yr,r.contrast=ge,r.cubehelix=fe,r.default=y,r.deltaE=Ae,r.distance=Ee,r.input=v,r.interpolate=Rr,r.limits=be,r.mix=Rr,r.random=ce,r.scale=Qr,r.scales=je,r.valid=Fe,Object.defineProperty(r,"__esModule",{value:!0})}));
